import os
import logging
import ConfigParser


def get_config(config_name='ricloud.ini'):
    config = ConfigParser.RawConfigParser()

    path_to_config = os.path.join(os.path.dirname(__file__), config_name)  # Read defaults.

    home_path = os.path.expanduser(os.path.join('~', '.%s' % config_name))  # Read user config.

    paths = [path_to_config, home_path]

    if 'RICLOUD_CONF' in os.environ:
        paths.append(os.environ['RICLOUD_CONF'])

    config.read(paths)

    return config


settings = get_config()

BASE_DIR = os.getcwd()

OUTPUT_DIR = os.path.join(BASE_DIR, settings.get('output', 'output_directory'))

LOG_DIRECTORY = os.path.join(BASE_DIR, settings.get('logging', 'logs_directory'))
LOG_FILE = os.path.join(LOG_DIRECTORY, 'ricloud.log')
LOG_LEVEL = settings.get('logging', 'level')

if not os.path.exists(LOG_DIRECTORY):
    os.makedirs(LOG_DIRECTORY)

logging.basicConfig(
    filename=LOG_FILE,
    filemode='a',
    format='[%(asctime)s: %(levelname)s/%(processName)s/%(thread)d] [%(name)s:%(lineno)s] %(message)s',
    level=LOG_LEVEL
)

LISTENER_DB_HOST = settings.get('mysql', 'host')
LISTENER_DB_PORT = settings.get('mysql', 'port')
LISTENER_DB_NAME = settings.get('mysql', 'database')
LISTENER_DB_USER = settings.get('mysql', 'username')
LISTENER_DB_PASSWORD = settings.get('mysql', 'password')

TEMPFILE_TIMEOUT = settings.getint('tempfiles', 'timeout')
TEMPFILE_TIMEOUT_CHECK_INTERVAL = settings.getint('tempfiles', 'timeout_check_interval')



import logging
import MySQLdb

from . import conf


logger = logging.getLogger(__name__)


class DatabaseHandler(object):

    def __init__(self, db_name=conf.LISTENER_DB_NAME):
        logger.debug('Setting up DatabaseHandler for database `%s`.', db_name)
        self.db_name = db_name

    _db_con = None

    @property
    def db_con(self):
        if not self._db_con:
            logger.debug('Establishing new database connection to database `%s`.', self.db_name)
            import MySQLdb
            self._db_con = MySQLdb.connect(
                host=conf.LISTENER_DB_HOST,
                port=int(conf.LISTENER_DB_PORT),
                user=conf.LISTENER_DB_USER,
                passwd=conf.LISTENER_DB_PASSWORD,
                db=self.db_name
            )
        return self._db_con

    def handle_query(self, query, args=None, retry=2):
        try:
            cursor = self.db_con.cursor()
            cursor.execute(query, args=args)
            self.db_con.commit()
        except (AttributeError, MySQLdb.OperationalError):
            if not retry:
                logger.error('Query failed, no retries remaining.', exc_info=True)
                raise

            logger.warn('Query failed, attempting to refresh connection (%d retries remaining)', retry)
            self._db_con = None
            self.handle_query(query, args=args, retry=retry - 1)



"""Base module for `ricloud`."""
from __future__ import absolute_import, unicode_literals

import logging
import threading
from cached_property import cached_property

from .conf import settings
from .api import Api
from .stream import Stream
from .object_store import ObjectStore
from .listener import Listener
from .handlers import RiCloudHandler
from .helpers import LogHelper


logger = logging.getLogger(__name__)


class RiCloud(object):
    """Primary class for dealing with the ricloud API/"""

    def __init__(self, api=None, listener=None):
        self.api = api or Api()
        self.api.setup()

        self.listener = listener or Listener({'__ALL__': RiCloudHandler(api=self.api)})

        self.services = self.api.allowed_services()

        if self.api.retrieval_protocol == "asstore":
            self.object_store_thread.start()
        else:
            self.stream_thread.start()

    _stream_thread = None

    @property
    def stream_thread(self):
        if not self._stream_thread:
            self._stream_thread = self._start_stream_thread()
        return self._stream_thread

    def _start_stream_thread(self):
        logger.info(LogHelper.get_message('stream_start'))
        stream = settings.get('stream', 'stream_endpoint')
        token = settings.get('auth', 'token')

        def _start():
            s = Stream(
                endpoint=self.api.stream_endpoints[0],
                listener=self.listener,
                stream=stream,
                token=token
            )
            s.go()

        stream_thread = threading.Thread(target=_start)
        stream_thread.daemon = True

        return stream_thread

    def _restart_stream_thread(self):
        logger.warn(LogHelper.get_message('stream_restart'))
        self._stream_thread = None
        self.stream_thread.start()

    def _check_stream_thread(self):
        if not self.stream_thread.is_alive():
            logger.warn(LogHelper.get_message('stream_failed_check'))
            self._restart_stream_thread()

    @cached_property
    def object_store_thread(self):

        def _start():
            s = ObjectStore(
                api=self.api
            )
            s.go()
        object_store_thread = threading.Thread(target=_start)
        object_store_thread.daemon = True
        return object_store_thread
